@@ -25,6 +25,9 @@
 G_BEGIN_DECLS
 
 void _tp_warn_failure (const char *op, GError *error);
+void _tp_register_marshallers (void);
+void _tp_conn_connect_req_handler (DBusGProxy *proxy, GError *error,
+    gpointer user_data);
 
 G_END_DECLS
 
