@@ -22,16 +22,22 @@
 
 #include <string.h>
 
+#include <telepathy-glib/dbus.h>
+
 #include "tp-helpers.h"
 #include "tp-connmgr.h"
 #include "tp-props-iface.h"
 #include "internal.h"
 #include "_gen/add-signals.h"
 
-static GObjectClass *parent_class = NULL;
+G_DEFINE_TYPE (TpConn, tp_conn, DBUS_TYPE_G_PROXY);
 
-static gboolean tp_conn_status_change_handler (DBusGProxy *proxy,
-    guint status, guint reason, gpointer user_data);
+typedef struct {
+    TpConnection *connection;
+} TpConnPrivate;
+
+#define TP_CONN_PRIV(o) \
+    (G_TYPE_INSTANCE_GET_PRIVATE (o, TELEPATHY_CONN_TYPE, TpConnPrivate))
 
 
 /* We initialize the list of signatures here, so that we can use
@@ -77,11 +83,8 @@ _tp_conn_init_interface_signal_signatures (GData **signal_sigs)
 
 
 static void
-tp_conn_init (GTypeInstance *instance,
-              gpointer g_class)
+tp_conn_init (TpConn *self)
 {
-  TpConn *self = TELEPATHY_CONN (instance);
-
   self->first_run = TRUE;
 }
 
@@ -92,11 +95,15 @@ static void tp_conn_dispose (GObject *obj)
 
   if (self->first_run == TRUE)
     {
+      TpConnPrivate *priv = TP_CONN_PRIV (self);
+
       self->first_run = FALSE;
       g_datalist_clear (&(self->interface_list));
+      g_object_unref (priv->connection);
+      priv->connection = NULL;
     }
 
-  G_OBJECT_CLASS (parent_class)->dispose (obj);
+  G_OBJECT_CLASS (tp_conn_parent_class)->dispose (obj);
 }
 
 
@@ -104,37 +111,14 @@ static void tp_conn_class_init (TpConnClass *klass)
 {
   GObjectClass *obj = G_OBJECT_CLASS (klass);
 
-  parent_class = g_type_class_peek_parent (klass);
+  g_type_class_add_private (klass, sizeof (TpConnPrivate));
+
   obj->dispose = tp_conn_dispose;
   _tp_register_marshallers ();
   _tp_conn_init_interface_signal_signatures (&(klass->iface_signal_sigs));
 }
 
 
-GType
-tp_conn_get_type (void)
-{
-  static GType type = 0;
-
-  if (G_UNLIKELY (type == 0))
-    {
-      static const GTypeInfo info = {
-          sizeof (TpConnClass),
-          NULL,
-          NULL,
-          (GClassInitFunc) tp_conn_class_init,
-          NULL,
-          NULL,
-          sizeof (TpConn),
-          0,
-          (GInstanceInitFunc) tp_conn_init
-      };
-
-      type = g_type_register_static (DBUS_TYPE_G_PROXY, "TpConn", &info, 0);
-    }
-  return type;
-}
-
 /* The interface name getters */
 
 GQuark
@@ -192,6 +176,118 @@ tp_get_conn_renaming_interface (void)
 }
 
 
+static gboolean tp_conn_status_change_handler (DBusGProxy *proxy,
+    guint status, guint reason, gpointer user_data);
+
+
+static TpConn *
+_tp_conn_new (TpConnection *connection,
+              guint *status,
+              GError **error)
+{
+  TpProxy *conn_as_proxy = (TpProxy *) connection;
+  gboolean connection_ready;
+  guint conn_status = TP_CONNECTION_STATUS_DISCONNECTED;
+  TpConn *obj;
+  TpConnPrivate *priv;
+  GError *err = NULL;
+
+  g_return_val_if_fail (connection != NULL, NULL);
+
+  /* Create the proxy object for this connection. It will be used to
+     perform the actual method calls over D-BUS. */
+
+  obj = g_object_new (TELEPATHY_CONN_TYPE,
+      "name", conn_as_proxy->bus_name,
+      "path", conn_as_proxy->object_path,
+      "interface", TP_IFACE_CONNECTION,
+      "connection", conn_as_proxy->dbus_connection,
+      NULL);
+
+  priv = TP_CONN_PRIV (obj);
+  priv->connection = g_object_ref (connection);
+
+  g_datalist_init (&(obj->interface_list));
+
+  ADD_SIGNALS_FOR_CONNECTION (DBUS_G_PROXY (obj));
+
+  g_object_get (connection,
+      "connection-ready", &connection_ready,
+      NULL);
+
+  if (connection_ready)
+    {
+      gchar **interfaces;
+
+      g_object_get (connection,
+          "status", &conn_status,
+          "interfaces", &interfaces,
+          NULL);
+
+      tp_conn_local_set_interfaces (obj, interfaces);
+      g_strfreev (interfaces);
+    }
+  else if (!tp_conn_get_status (DBUS_G_PROXY (obj), &conn_status, &err))
+    {
+      g_propagate_error (error, err);
+      g_object_unref (obj);
+      return NULL;
+    }
+  else if (conn_status == TP_CONNECTION_STATUS_CONNECTED)
+    {
+      tp_conn_status_change_handler (DBUS_G_PROXY (obj), conn_status,
+          0, NULL);
+    }
+  else
+    {
+      /* We can't use connection-ready because of our API guarantee that
+       * by the time a later StatusChanged handler runs, we'll be ready -
+       * connection-ready occurs some time after StatusChanged, because it
+       * uses async calls */
+      dbus_g_proxy_connect_signal (DBUS_G_PROXY (obj), "StatusChanged",
+          G_CALLBACK (tp_conn_status_change_handler), NULL, NULL);
+    }
+
+  if (status != NULL)
+    *status = conn_status;
+
+  return obj;
+}
+
+
+TpConn *
+tp_conn_new_from_connection (TpConnection *connection,
+                             GError **error)
+{
+  return _tp_conn_new (connection, NULL, error);
+}
+
+
+/* Assume that in practice people will always use the same DBusGConnection. */
+static TpDBusDaemon *
+_get_dbus_daemon (DBusGConnection *connection)
+{
+  static TpDBusDaemon *last_daemon = NULL;
+  static DBusGConnection *last_connection = NULL;
+
+  if (last_connection == connection &&
+      last_daemon != NULL)
+    return last_daemon;
+
+  if (last_connection != NULL)
+    dbus_g_connection_unref (last_connection);
+
+  last_connection = dbus_g_connection_ref (connection);
+
+  if (last_daemon != NULL)
+    g_object_unref (last_daemon);
+
+  last_daemon = tp_dbus_daemon_new (last_connection);
+
+  return last_daemon;
+}
+
+
 TpConn *
 tp_conn_new_without_connect (DBusGConnection *connection,
                              const gchar *bus_name,
@@ -199,79 +295,15 @@ tp_conn_new_without_connect (DBusGConnection *connection,
                              guint *status,
                              GError **error)
 {
-  gchar *unique_name;
-  gchar **interfaces;
-  guint conn_status = TP_CONNECTION_STATUS_DISCONNECTED;
-  TpConn *obj;
-  GError *err = NULL;
+  TpConnection *conn;
+  TpDBusDaemon *dbus_daemon = _get_dbus_daemon (connection);
 
-  g_return_val_if_fail (connection != NULL, NULL);
-  g_return_val_if_fail (bus_name != NULL, NULL);
-  g_return_val_if_fail (object_path, NULL);
+  conn = tp_connection_new (dbus_daemon, bus_name, object_path, error);
 
-  /* Create the proxy object for this connection. It will be used to
-     perform the actual method calls over D-BUS. */
+  if (conn == NULL)
+    return NULL;
 
-  if (!dbus_g_proxy_call (tp_get_bus_proxy (), "GetNameOwner", error,
-        G_TYPE_STRING, bus_name, G_TYPE_INVALID,
-        G_TYPE_STRING, &unique_name, G_TYPE_INVALID))
-    {
-      return NULL;
-    }
-
-  obj = g_object_new (TELEPATHY_CONN_TYPE,
-      "name", unique_name,
-      "path", object_path,
-      "interface", TP_IFACE_CONNECTION,
-      "connection", connection,
-      NULL);
-  g_free (unique_name);
-
-  g_datalist_init (&(obj->interface_list));
-
-  ADD_SIGNALS_FOR_CONNECTION (DBUS_G_PROXY (obj));
-
-  /* Check if the connection is already connected. If so, we can
-   * already perform GetInterfaces(). */
-
-  if (!tp_conn_get_status (DBUS_G_PROXY (obj), &conn_status, &err))
-    {
-      if (err != NULL)
-        g_propagate_error (error, err);
-
-      g_object_unref (obj);
-      return NULL;
-    }
-
-  if (conn_status == TP_CONNECTION_STATUS_CONNECTED)
-    {
-      if (!tp_conn_get_interfaces (DBUS_G_PROXY (obj), &interfaces, &err))
-        {
-          if (err != NULL)
-            g_propagate_error (error, err);
-
-          g_object_unref (obj);
-          return NULL;
-        }
-
-      /* Initialize the interface objects for this TpConn object */
-      tp_conn_local_set_interfaces (obj, interfaces);
-
-      g_strfreev (interfaces);
-    }
-  else
-    {
-      /* Not connected yet, so this is really a new connection. Thus, we
-         have to hook up to StatusChanged signal to perform the
-         GetInterfaces when it goes Connected */
-
-      dbus_g_proxy_connect_signal (DBUS_G_PROXY (obj), "StatusChanged",
-          G_CALLBACK (tp_conn_status_change_handler), NULL, NULL);
-    }
-
-  if (status != NULL)
-    *status = conn_status;
-  return obj;
+  return _tp_conn_new (conn, status, error);
 }
 
 
@@ -328,9 +360,10 @@ tp_conn_new_channel (DBusGConnection *connection,
                      guint handle,
                      gboolean supress_handler)
 {
+  TpConnPrivate *priv = TP_CONN_PRIV (tp_conn);
   GError *error = NULL;
   gchar *chan_object_path = NULL;
-  TpChan *new_chan = NULL;
+  TpChannel *channel;
 
   g_return_val_if_fail (connection, NULL);
   g_return_val_if_fail (TELEPATHY_IS_CONN (tp_conn), NULL);
@@ -353,11 +386,22 @@ tp_conn_new_channel (DBusGConnection *connection,
 
   /* Create the object to represent the channel */
 
-  new_chan = tp_chan_new (connection, bus_name, chan_object_path, type,
-      handle_type, handle);
+  channel = tp_channel_new (priv->connection, chan_object_path, NULL,
+      TP_UNKNOWN_HANDLE_TYPE, 0, &error);
+
   g_free (chan_object_path);
 
-  return new_chan;
+  if (channel == NULL)
+    {
+      _tp_warn_failure ("RequestChannel()", error);
+
+      if (error != NULL)
+        g_error_free (error);
+
+      return NULL;
+    }
+
+  return tp_chan_new_from_channel (channel);
 }
 
 
@@ -489,3 +533,11 @@ tp_conn_status_change_handler (DBusGProxy *proxy,
   return TRUE;
 }
 
+
+TpConnection *
+tp_conn_dup_connection (TpConn *self)
+{
+  TpConnPrivate *priv = TP_CONN_PRIV (self);
+
+  return g_object_ref (priv->connection);
+}
