@@ -1,6 +1,7 @@
 /* tp-conn.h
  *
- * Copyright (C) 2005 Nokia Corporation.
+ * Copyright (C) 2005-2007 Nokia Corporation
+ * Copyright (C) 2006-2007 Collabora Ltd. <http://www.collabora.co.uk/>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,8 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
- *
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -26,6 +26,9 @@
 #define DBUS_API_SUBJECT_TO_CHANGE
 #endif
 
+#include <dbus/dbus-glib.h>
+#include <telepathy-glib/connection.h>
+
 #include "tp-conn-gen.h"
 #include "tp-conn-iface-forwarding-gen.h"
 #include "tp-chan.h"
@@ -35,39 +38,48 @@
 #include "tp-interfaces.h"
 #include "tp-constants.h"
 
-#include <dbus/dbus-glib.h>
-
 #define DBUS_TYPE_G_ARRAY_OF_STRUCTS dbus_g_type_get_collection ("GPtrArray", G_TYPE_VALUE_ARRAY);
 
 #define TELEPATHY_CONN_TYPE  (tp_conn_get_type ())
 
-#define TELEPATHY_CONN(obj)  (G_TYPE_CHECK_INSTANCE_CAST \
-		                    ((obj), TELEPATHY_CONN_TYPE, \
-				     TpConn))
+#define TELEPATHY_CONN(obj) (G_TYPE_CHECK_INSTANCE_CAST \
+    ((obj), TELEPATHY_CONN_TYPE, TpConn))
 
-#define TELEPATHY_CONN_CLASS(klass)  (G_TYPE_CHECK_CLASS_CAST \
-		                          ((klass), TELEPATHY_CONN_TYPE, \
-					  TpConnClass))
+#define TELEPATHY_CONN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST \
+    ((klass), TELEPATHY_CONN_TYPE, TpConnClass))
 
-#define TELEPATHY_IS_CONN(obj)    (G_TYPE_CHECK_INSTANCE_TYPE \
-		                      ((obj), TELEPATHY_CONN_TYPE))
+#define TELEPATHY_IS_CONN(obj) (G_TYPE_CHECK_INSTANCE_TYPE \
+    ((obj), TELEPATHY_CONN_TYPE))
 
-#define TELEPATHY_IS_CONN_CLASS(klass)  (G_TYPE_CHECK_CLASS_TYPE \
-		                           ((klass), TELEPATHY_CONN_TYPE))
+#define TELEPATHY_IS_CONN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE \
+    ((klass), TELEPATHY_CONN_TYPE))
 
-#define TELEPATHY_CONN_GET_CLASS(obj)   (G_TYPE_INSTANCE_GET_CLASS \
-		                            ((obj), TELEPATHY_CONN_TYPE, \
-					    TpConnClass))
+#define TELEPATHY_CONN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS \
+    ((obj), TELEPATHY_CONN_TYPE, TpConnClass))
 
-#define TELEPATHY_CONN_IFACE_QUARK (tp_get_conn_interface())
-#define TELEPATHY_CONN_IFACE_ALIASING_QUARK (tp_get_conn_aliasing_interface())
-#define TELEPATHY_CONN_IFACE_AVATARS_QUARK (tp_get_conn_avatars_interface())
-#define TELEPATHY_CONN_IFACE_CAPABILITIES_QUARK (tp_get_conn_capabilities_interface())
-#define TELEPATHY_CONN_IFACE_CONTACTINFO_QUARK (tp_get_conn_contactinfo_interface())
-#define TELEPATHY_CONN_IFACE_FORWARDING_QUARK (tp_get_conn_forwarding_interface())
-#define TELEPATHY_CONN_IFACE_PRESENCE_QUARK (tp_get_conn_presence_interface())
-#define TELEPATHY_CONN_IFACE_PRIVACY_QUARK (tp_get_conn_privacy_interface())
-#define TELEPATHY_CONN_IFACE_RENAMING_QUARK (tp_get_conn_renaming_interface())
+#ifndef LIBTELEPATHY_DISABLE_DEPRECATED
+
+/* For backwards compatibility only. Use TP_IFACE_QUARK_* in new code */
+#define TELEPATHY_CONN_IFACE_QUARK \
+  TP_IFACE_QUARK_CONNECTION
+#define TELEPATHY_CONN_IFACE_ALIASING_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_ALIASING
+#define TELEPATHY_CONN_IFACE_AVATARS_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_AVATARS
+#define TELEPATHY_CONN_IFACE_CAPABILITIES_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_CAPABILITIES
+#define TELEPATHY_CONN_IFACE_CONTACTINFO_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_INFO
+#define TELEPATHY_CONN_IFACE_FORWARDING_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_FORWARDING
+#define TELEPATHY_CONN_IFACE_PRESENCE_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_PRESENCE
+#define TELEPATHY_CONN_IFACE_PRIVACY_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_PRIVACY
+#define TELEPATHY_CONN_IFACE_RENAMING_QUARK \
+  TP_IFACE_QUARK_CONNECTION_INTERFACE_RENAMING
+
+#endif
 
 typedef struct _tp_conn TpConn;
 typedef struct _tp_connclass TpConnClass;
@@ -87,29 +99,58 @@ struct _tp_connclass
 };
 
 
-GType tp_conn_get_type(void);
+GType tp_conn_get_type (void);
 
 /* Actual function definitions */
 
 /*
- * Create a new TpConn (Connection) object.
+ * Create a new TpConn (Connection) object and try to connect asynchronously.
  *
  * @param connection The D-BUS connection
  * @param name: The D-BUS service name for the connection object
  * @param path_name: The D-BUS path name for the connection object
  * @param interface_name: The D-BUS interface name for the connection object
  * @return A new TpConn connection object, or NULL if unsuccesful.
+ *
+ * @deprecated since 0.3.0. Use tp_conn_new_without_connect() instead
  */
 
-TpConn *tp_conn_new(DBusGConnection *connection,
-		    const char      *bus_name,
-		    const char      *object_path);
+#ifndef LIBTELEPATHY_DISABLE_DEPRECATED
+TpConn *tp_conn_new (DBusGConnection *connection, const char *bus_name,
+    const char *object_path) G_GNUC_DEPRECATED;
+#endif
+
+/* Create a new #TpConn from a #TpConnection.
+ * It will be ready for use if and only if the #TpConnection is; consider
+ * using tp_connection_run_until_ready() first.
+ *
+ * @param connection: A telepathy-glib #TpConnection
+ * @param error: Used to return error if GetStatus() fails
+ * @return A new TpConn object, or NULL if GetStatus() fails
+ */
+TpConn *tp_conn_new_from_connection (TpConnection *connection, GError **error);
+
+/*
+ * Retrieve or create a telepathy-glib TpConnection object for this connection.
+ *
+ * The returned TpConnection is not guaranteed to be ready (use the
+ * connection-ready property, the notify::connection-ready signal, or
+ * tp_connection_run_until_ready()).
+ *
+ * This function exists to aid migration from libtelepathy to telepathy-glib.
+ * Available since 0.3.2.
+ *
+ * @param self: the libtelepathy connection object
+ * @return a new reference to a TpConnection, which must be unreffed by the
+ *  caller
+ */
+TpConnection *tp_conn_dup_connection (TpConn *self);
 
 /* Create a new #TpConn and connect signal handlers, but do not try to connect
  * asynchronously.
  *
- * If TP_CONN_STATUS_CONNECTED is written through @status, the new connection
- * is connected, and has its interfaces set up and is ready for use.
+ * If TP_CONNECTION_STATUS_CONNECTED is written through @status, the new
+ * connection is connected, and has its interfaces set up and is ready for use.
  * Otherwise, the caller is responsible for calling Connect and waiting for
  * the connection attempt to either succeed or fail.
  *
@@ -142,35 +183,24 @@ TpConn *tp_conn_new_without_connect (DBusGConnection *connection,
  *                         launched
  *
  * @return A new TpChan (channel) object
- */ 
+ */
 
-TpChan *tp_conn_new_channel(DBusGConnection *connection, TpConn *tpconn,
-			    const gchar *bus_name,
-			    gchar *type, guint handle_type,
-			    guint handle, gboolean supress_handler);
+TpChan *tp_conn_new_channel (DBusGConnection *connection, TpConn *tpconn,
+    const gchar *bus_name, gchar *type, guint handle_type, guint handle,
+    gboolean supress_handler);
 
 
 /* Creates the proxy objects for the channel interfaces. GetInterfaces
  * method should be called before this to acquire the interface name
  * list.
- * 
+ *
  * @param self: The connection object
  * @param interfaces: An array of the interface name strings
  */
 
-void tp_conn_local_set_interfaces(TpConn *self, gchar **interfaces);
+void tp_conn_local_set_interfaces (TpConn *self, gchar **interfaces);
 
 
-/*
- * Provides access to the connection proxy object to use for actual
- * method calls
- *
- * @param self: The connection object whose proxy should be retrieved
- * @return The D-BUS proxy object
- */
-
-DBusGProxy *tp_conn_get_proxy(TpConn *self);
-
 /*
  * This function checks whether the Connection has the specified
  * interface and returns a proxy object that can be used to call
@@ -184,17 +214,18 @@ DBusGProxy *tp_conn_get_proxy(TpConn *self);
  * @return A DBusGProxy object for the interface, or NULL if not found
  */
 
-DBusGProxy *tp_conn_get_interface(TpConn *self, GQuark iface_quark);
-
-GQuark tp_get_conn_interface(void);
-GQuark tp_get_conn_aliasing_interface(void);
-GQuark tp_get_conn_avatars_interface(void);
-GQuark tp_get_conn_capabilities_interface(void);
-GQuark tp_get_conn_contactinfo_interface(void);
-GQuark tp_get_conn_forwarding_interface(void);
-GQuark tp_get_conn_presence_interface(void);
-GQuark tp_get_conn_privacy_interface(void);
-GQuark tp_get_conn_renaming_interface(void);
+DBusGProxy *tp_conn_get_interface (TpConn *self, GQuark iface_quark);
 
+#ifndef LIBTELEPATHY_DISABLE_DEPRECATED
+GQuark tp_get_conn_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_aliasing_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_avatars_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_capabilities_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_contactinfo_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_forwarding_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_presence_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_privacy_interface (void) G_GNUC_DEPRECATED;
+GQuark tp_get_conn_renaming_interface (void) G_GNUC_DEPRECATED;
+#endif
 
 #endif
